@@ -1,4 +1,3 @@
-
 package PDL::Options;
 
 =head1 NAME
@@ -37,10 +36,9 @@ A simplified (non-OO) interface is provided.
 =cut
 
 use strict;
+use warnings;
 use Carp;
 
-use vars qw/$VERSION %EXPORT_TAGS %DEF_SYNS @ISA/;
-
 require Exporter;
 
 # difference to 0.91 is that added CENTRE/CENTER as default
@@ -48,9 +46,9 @@ require Exporter;
 our $VERSION = '0.92';
 $VERSION = eval $VERSION;
 
-@ISA = qw(Exporter);
+our @ISA = qw(Exporter);
 
-%EXPORT_TAGS = (
+our %EXPORT_TAGS = (
 		'Func' => [qw/
 			   parse iparse ifhref
 			   /]
@@ -59,7 +57,7 @@ $VERSION = eval $VERSION;
 Exporter::export_tags('Func');
 
 # List of default synonyms
-%DEF_SYNS = (
+our %DEF_SYNS = (
 	     COLOR  => 'COLOUR',
 	     COLOUR => 'COLOR',
 	     CENTER => 'CENTRE',
@@ -718,11 +716,11 @@ sub options {
     foreach my $userkey (sort keys %user) {
 
       # Check for matches in the default set
-      my @matched = $self->compare_with_list(0, $userkey, keys %base);
+      my @matched = $self->compare_with_list(0, $userkey, sort keys %base);
 
       # If we had no matches, check the synonyms list
       if ($#matched == -1) {
-	@matched = $self->compare_with_list(0, $userkey, keys %syn);
+	@matched = $self->compare_with_list(0, $userkey, sort keys %syn);
 
 	# If we have matched then convert the key to the actual
 	# value stored in the object
@@ -793,14 +791,14 @@ sub translate {
   # Now need to go through each of the keys
   # and if the corresponding key exists in the translation
   # hash we need to check that a valid translation exists
-  foreach my $key ( keys %opt ) {
+  foreach my $key ( sort keys %opt ) {
     if (exists $trans{$key}) {
       # Okay so a translation might exist
       # Now compare keys in the hash in the hash
       my %subhash = %{$trans{$key}};
 
       my @matched =
-	$self->compare_with_list(1, $opt{$key}, keys %subhash);
+	$self->compare_with_list(1, $opt{$key}, sort keys %subhash);
 
       # At this point we have matched the userkey to a key in the
       # dictionary. If there is no translation dont say anything
@@ -890,10 +888,7 @@ sub compare_with_list {
 	# exact match even if alternatives exist (eg COL will always
 	# match just COL if the keys are COL and COLOUR)
 	# First do the exact match (case insensitive)
-      {
-	local $^W = undef; # To silence warnings about uninitialised values
 	@result =  grep { /^$key$/i } @list;
-      }
 	# If this match came up with something then we will use it
 	# Else we will try a minimum match (assuming flag is true)
 
@@ -907,9 +902,6 @@ sub compare_with_list {
     return @result;
 }
 
-
-
-
 =back
 
 =head1 EXAMPLE
@@ -930,7 +922,7 @@ the second uses the object-oriented interface.
 		  }
 		);
 
-This will return a hash containg
+This will return a hash containing
 
     %options = (
                  LINE => 1,
