@@ -1,31 +1,35 @@
-#
-#
 # ToDo:
 #  - multiple windows - requires editing generate.pl in OpenGL/
 #  - clean up
 #
 #package PDL::Graphics::TriD::GL;
 
-use PDL::Graphics::OpenGL;
-use PDL::Graphics::OpenGLQ;
+use strict;
+use warnings;
+no warnings 'redefine';
+use OpenGL qw/ :glfunctions :glconstants gluPerspective gluOrtho2D /;
+use OpenGL::GLUT qw( :all );
+use PDL::Graphics::OpenGL::Perl::OpenGL;
 
+$PDL::Graphics::TriD::create_window_sub = # warnings
 $PDL::Graphics::TriD::create_window_sub = sub {
-  return new PDL::Graphics::TriD::GL::Window(@_);
+  return PDL::Graphics::TriD::GL::Window->new(@_);
 };
 
 sub PDL::Graphics::TriD::Material::togl{
   my $this = shift;
   my $shin = pack "f*",$this->{Shine};
-  glMaterialfv(&GL_FRONT_AND_BACK,&GL_SHININESS,$shin);
+  glMaterialfv(GL_FRONT_AND_BACK,GL_SHININESS,$shin);
   my $spec = pack "f*",@{$this->{Specular}};
-  glMaterialfv(&GL_FRONT_AND_BACK,&GL_SPECULAR,$spec);
+  glMaterialfv(GL_FRONT_AND_BACK,GL_SPECULAR,$spec);
   my $amb = pack "f*",@{$this->{Ambient}};
   glMaterialfv(GL_FRONT_AND_BACK,GL_AMBIENT,$amb);
   my $diff = pack "f*",@{$this->{Diffuse}};
-  glMaterialfv(&GL_FRONT_AND_BACK,&GL_DIFFUSE,$diff);
+  glMaterialfv(GL_FRONT_AND_BACK,GL_DIFFUSE,$diff);
 }
 
 $PDL::Graphics::TriD::any_cannots = 0;
+$PDL::Graphics::TriD::verbose //= 0;
 
 sub PDL::Graphics::TriD::Object::cannot_mklist {
 	return 0;
@@ -96,20 +100,20 @@ sub PDL::Graphics::TriD::Object::togl {
 sub PDL::Graphics::TriD::BoundingBox::togl { 
   my($this) = @_;
   $this = $this->{Box};
-  glDisable(&GL_LIGHTING);
+  glDisable(GL_LIGHTING);
   glColor3d(1,1,1);
-  glBegin(&GL_LINES);
+  glBegin(GL_LINES);
   for([0,4,2],[0,1,2],[0,1,5],[0,4,5],[0,4,2],[3,4,2],
 		[3,1,2],[3,1,5],[3,4,5],[3,4,2]) {
 	 &glVertex3d(@{$this}[@$_]);
   }
   glEnd();
-  glBegin(&GL_LINE_STRIP);
+  glBegin(GL_LINE_STRIP);
   for([0,1,2],[3,1,2],[0,1,5],[3,1,5],[0,4,5],[3,4,5]) {
 	 &glVertex3d(@{$this}[@$_]);
   }
   glEnd();
-  glEnable(&GL_LIGHTING);
+  glEnable(GL_LIGHTING);
 }
 
 sub PDL::Graphics::TriD::Graph::togl {
@@ -135,7 +139,7 @@ sub PDL::Graphics::TriD::CylindricalEquidistantAxes::togl_axis {
    
         my (@nadd,@nc,@ns);
 
-	for $dim (0..1) {
+	for my $dim (0..1) {
 	  my $width = $this->{Scale}[$dim][1]-$this->{Scale}[$dim][0];
 	  if($width > 100){
 	    $nadd[$dim] = 10;
@@ -153,7 +157,7 @@ sub PDL::Graphics::TriD::CylindricalEquidistantAxes::togl_axis {
 	# can be changed to topo heights?
 	my $verts = zeroes(3,$ns[0],$ns[1]);
 
-	($t = $verts->slice("2")) .= 1012.5;
+	(my $t = $verts->slice("2")) .= 1012.5;
 	($t = $verts->slice("0")) .= $verts->ylinvals($nc[0],$nc[0]+$nadd[0]*($ns[0]-1));
 	($t = $verts->slice("1")) .= $verts->zlinvals($nc[1],$nc[1]+$nadd[1]*($ns[1]-1));
 
@@ -162,11 +166,11 @@ sub PDL::Graphics::TriD::CylindricalEquidistantAxes::togl_axis {
 
 	$tverts = $this->transform($tverts,$verts,[0,1,2]);
 
-	glDisable(&GL_LIGHTING);
+	glDisable(GL_LIGHTING);
 	glColor3d(1,1,1);
 	for(my $j=0;$j<$tverts->getdim(2)-1;$j++){
 	  my $j1=$j+1;
-	  glBegin(&GL_LINES);
+	  glBegin(GL_LINES);
 	  for(my $i=0;$i<$tverts->getdim(1)-1;$i++){
 	    my $i1=$i+1;
 
@@ -187,7 +191,7 @@ sub PDL::Graphics::TriD::CylindricalEquidistantAxes::togl_axis {
 	  glEnd();
 	}
 
-	glEnable(&GL_LIGHTING);
+	glEnable(GL_LIGHTING);
 }
 
 
@@ -196,13 +200,14 @@ sub PDL::Graphics::TriD::CylindricalEquidistantAxes::togl_axis {
 sub PDL::Graphics::TriD::EuclidAxes::togl_axis {
 	my($this,$graph) = @_;
 
+        print "togl_axis: got object type " . ref($this) . "\n" if $PDL::debug_trid;
 #	print "TOGLAX\n";
 	my $fontbase = $PDL::Graphics::TriD::GL::fontbase;
 #	print "TOGL EUCLID\n";
    glLineWidth(1); # ought to be user defined
-	glDisable(&GL_LIGHTING);
+	glDisable(GL_LIGHTING);
 	glColor3d(1,1,1);
-	glBegin(&GL_LINES);
+	glBegin(GL_LINES);
 	my $dim;
 	for $dim (0..2) {
 		glVertex3f(0,0,0);
@@ -223,9 +228,12 @@ sub PDL::Graphics::TriD::EuclidAxes::togl_axis {
 		my $nc = $s->[0];
 		for(0..$ndiv) {
 			&glRasterPos3f(@coords);
-			PDL::Graphics::OpenGL::glpPrintString($fontbase,
-				sprintf("%.3f",$nc));
-			glBegin(&GL_LINES);
+			if ( done_glutInit() ) {
+			   glutBitmapString($fontbase, sprintf("%.3f",$nc));
+			} else {
+			   OpenGL::glpPrintString($fontbase, sprintf("%.3f",$nc));
+			}
+			glBegin(GL_LINES);
 			&glVertex3f(@coords0);
 			&glVertex3f(@coords);
 			glEnd();
@@ -236,15 +244,18 @@ sub PDL::Graphics::TriD::EuclidAxes::togl_axis {
 		}
 		$coords0[$dim] = 1.1;
 		&glRasterPos3f(@coords0);
-		PDL::Graphics::OpenGL::glpPrintString($fontbase,
-			$this->{Names}[$dim]);
-	}
-	glEnable(&GL_LIGHTING);
+		if ( done_glutInit() ) {
+			glutBitmapString($fontbase, $this->{Names}[$dim]);
+		} else {
+			OpenGL::glpPrintString($fontbase, $this->{Names}[$dim]);
+		}
+             }
+	glEnable(GL_LIGHTING);
 }
 
 
 
-use POSIX qw/acos/;
+use POSIX qw//;
 sub PDL::Graphics::TriD::Quaternion::togl {
   my($this) = @_;
   if(abs($this->[0]) == 1) { return ; }
@@ -252,7 +263,7 @@ sub PDL::Graphics::TriD::Quaternion::togl {
     # die "Unnormalized Quaternion!\n";
     $this->normalize_this();
   } 
-  &glRotatef(2*acos($this->[0])/3.14*180, @{$this}[1..3]);
+  &glRotatef(2*POSIX::acos($this->[0])/3.14*180, @{$this}[1..3]);
 }
 
 ##################################
@@ -275,19 +286,37 @@ sub PDL::Graphics::TriD::Points::gdraw {
 #	print "DRAWPOINTS: \n",$points;
 
 	$this->glOptions();
-	glDisable(&GL_LIGHTING);
+	glDisable(GL_LIGHTING);
 	PDL::gl_points($points,$this->{Colors});
-	glEnable(&GL_LIGHTING);
+	glEnable(GL_LIGHTING);
+}
+
+sub PDL::gl_spheres { 
+   my ($coords,$colors) = @_;   
+   for (my $np=0; $np<$coords->dim(1); $np++) {
+      glPushMatrix();
+      my ($x,$y,$z) = ($coords->slice(":,($np)"))->float->list;
+      glTranslatef($x,$y,$z);
+      glutSolidSphere(0.025,15,15);
+      glPopMatrix();
+   }
+}
+
+sub PDL::Graphics::TriD::Spheres::gdraw {
+   my($this,$points) = @_;
+   $this->glOptions();
+   glShadeModel(GL_SMOOTH);
+   PDL::gl_spheres($points,$this->{Colors});
 }
 
 sub PDL::Graphics::TriD::Lattice::gdraw {
 	my($this,$points) = @_;
 
 	$this->glOptions();
-	glDisable(&GL_LIGHTING);
+	glDisable(GL_LIGHTING);
 	PDL::gl_line_strip($points,$this->{Colors});
 	PDL::gl_line_strip($points->xchg(1,2),$this->{Colors}->xchg(1,2));
-	glEnable(&GL_LIGHTING);
+	glEnable(GL_LIGHTING);
 }
 
 
@@ -295,18 +324,18 @@ sub PDL::Graphics::TriD::LineStrip::gdraw {
 	my($this,$points) = @_;
 
 	$this->glOptions();
-	glDisable(&GL_LIGHTING);
+	glDisable(GL_LIGHTING);
 	PDL::gl_line_strip($points,$this->{Colors});
-	glEnable(&GL_LIGHTING);
+	glEnable(GL_LIGHTING);
 }
 
 sub PDL::Graphics::TriD::Lines::gdraw {
 	my($this,$points) = @_;
 
 	$this->glOptions();
-	glDisable(&GL_LIGHTING);
+	glDisable(GL_LIGHTING);
 	PDL::gl_lines($points,$this->{Colors});
-	glEnable(&GL_LIGHTING);
+	glEnable(GL_LIGHTING);
 }
 
 sub PDL::Graphics::TriD::GObject::glOptions {
@@ -336,7 +365,7 @@ sub PDL::Graphics::TriD::Contours::gdraw {
 
   $this->glOptions();
 
-  glDisable(&GL_LIGHTING);
+  glDisable(GL_LIGHTING);
   my $pcnt=0;
   my $i=0;
 
@@ -361,7 +390,7 @@ sub PDL::Graphics::TriD::Contours::gdraw {
 												 ,$this->{LabelStrings});
   }
   
-  glEnable(&GL_LIGHTING);
+  glEnable(GL_LIGHTING);
 }
 
 sub PDL::Graphics::TriD::SLattice::gdraw {
@@ -369,10 +398,10 @@ sub PDL::Graphics::TriD::SLattice::gdraw {
 
 	$this->glOptions();
 
-	glPushAttrib(&GL_LIGHTING_BIT | &GL_ENABLE_BIT);
-	glDisable(&GL_LIGHTING);
+	glPushAttrib(GL_LIGHTING_BIT | GL_ENABLE_BIT);
+	glDisable(GL_LIGHTING);
 # By-vertex doesn't make sense otherwise.
-	glShadeModel (&GL_SMOOTH);
+	glShadeModel (GL_SMOOTH);
 	my @sls1 = (":,0:-2,0:-2",
 	            ":,1:-1,0:-2",
 		    ":,0:-2,1:-1");
@@ -401,10 +430,10 @@ sub PDL::Graphics::TriD::SCLattice::gdraw {
 
 	$this->glOptions();
 
-	glPushAttrib(&GL_LIGHTING_BIT | &GL_ENABLE_BIT);
-	glDisable(&GL_LIGHTING);
+	glPushAttrib(GL_LIGHTING_BIT | GL_ENABLE_BIT);
+	glDisable(GL_LIGHTING);
 # By-vertex doesn't make sense otherwise.
-	glShadeModel (&GL_FLAT);
+	glShadeModel (GL_FLAT);
 	my @sls1 = (":,0:-2,0:-2",
 	            ":,1:-1,0:-2",
 		    ":,0:-2,1:-1");
@@ -434,11 +463,11 @@ sub PDL::Graphics::TriD::SLattice_S::gdraw {
 
 	$this->glOptions();
 
-	glPushAttrib(&GL_LIGHTING_BIT | &GL_ENABLE_BIT);
+	glPushAttrib(GL_LIGHTING_BIT | GL_ENABLE_BIT);
 # For some reason, we need to set this here as well.
-	glLightModeli(&GL_LIGHT_MODEL_TWO_SIDE, &GL_TRUE);
+	glLightModeli(GL_LIGHT_MODEL_TWO_SIDE, GL_TRUE);
 # By-vertex doesn't make sense otherwise.
-	glShadeModel (&GL_SMOOTH);
+	glShadeModel (GL_SMOOTH);
 	my @sls1 = (":,0:-2,0:-2",
 	            ":,1:-1,0:-2",
 		    ":,0:-2,1:-1");
@@ -474,7 +503,7 @@ sub PDL::Graphics::TriD::SLattice_S::gdraw {
 			      (map {$this->{Colors}->slice($_)} @sls2)
 			     );
 	}
-	glDisable(&GL_LIGHTING);
+	glDisable(GL_LIGHTING);
 	if ($this->{Options}{Lines}) {
 	  my $black = PDL->pdl(0,0,0)->dummy(1)->dummy(1);
 	  PDL::gl_line_strip($points,$black);
@@ -483,13 +512,62 @@ sub PDL::Graphics::TriD::SLattice_S::gdraw {
 	glPopAttrib();
 }
 
+#################################################################### 
+################### JNK 15mar11 added section start ################
+sub PDL::Graphics::TriD::STrigrid_S::gdraw {
+  my($this,$points) = @_;
+  glPushAttrib(GL_LIGHTING_BIT | GL_ENABLE_BIT);
+  # For some reason, we need to set this here as well.
+  glLightModeli(GL_LIGHT_MODEL_TWO_SIDE, GL_TRUE);
+  # By-vertex doesn't make sense otherwise.
+  glShadeModel (GL_SMOOTH);   
+  my @sls = (":,(0)",":,(1)",":,(2)");
+  my $idx = [0,1,2,0]; # for lines, below
+  if ($this->{Options}{Smooth}) {
+    $this->{Normals}=$this->smoothn($this->{Points})
+      unless defined($this->{Normals}); 
+    my $f=(!$this->{Options}{Material}?\&PDL::gl_triangles_wn
+                                      :\&PDL::gl_triangles_wn_mat);
+    my $tmpn=$this->{Normals}->dice_axis(1,$this->{Faceidx}->clump(-1))
+                    ->splitdim(1,($this->{Faceidx}->dims)[0]);
+    my @args=((map {$this->{Faces}->slice($_)} @sls),   # faces is a slice of points
+              (map {$tmpn->slice($_)} @sls),
+              (map {$this->{Colors}->slice($_)} @sls) );&$f(@args); }
+  else {
+    my $f=(!$this->{Options}{Material}?\&PDL::gl_triangles_n
+                                      :\&PDL::gl_triangles_n_mat);
+    &$f( (map {$this->{Faces}->slice($_)} @sls),   # faces is a slice of points
+         (map {$this->{Colors}->slice($_)} @sls) ); } 
+  glDisable(GL_LIGHTING);
+  if ($this->{Options}{Lines}) {
+    my $black = PDL->pdl(0,0,0)->dummy(1)->dummy(1);
+    PDL::gl_lines($this->{Faces}->dice_axis(1,$idx),$black); } glPopAttrib(); }
+
+sub PDL::Graphics::TriD::STrigrid::gdraw {
+  my($this,$points) = @_;
+  glPushAttrib(GL_LIGHTING_BIT | GL_ENABLE_BIT);
+  glDisable(GL_LIGHTING);
+# By-vertex doesn't make sense otherwise.
+  glShadeModel (GL_SMOOTH);
+  my @sls = (":,(0)",":,(1)",":,(2)");
+  my $idx = [0,1,2,0];
+  PDL::gl_triangles(
+    (map {$this->{Faces}->slice($_)} @sls),   # faces is a slice of points
+    (map {$this->{Colors}->slice($_)} @sls));
+  if ($this->{Options}{Lines}) {
+    my $black = PDL->pdl(0,0,0)->dummy(1)->dummy(1);
+    PDL::gl_lines($this->{Faces}->dice_axis(1,$idx),$black); }
+  glPopAttrib(); }
+################### JNK 15mar11 added section finis ################
+####################################################################
+
 ##################################
 # PDL::Graphics::TriD::Image
 #
 #
 
 sub PDL::Graphics::TriD::Image::togl {
-#  glDisable(&GL_LIGHTING);
+#  glDisable(GL_LIGHTING);
 #
 # A special construct which always faces the display and takes the entire window
 #  
@@ -512,25 +590,16 @@ sub PDL::Graphics::TriD::Image::gdraw {
 	my($this,$vert) = @_;
 	my ($p,$xd,$yd,$txd,$tyd) = $this->flatten(1); # do binary alignment
 	glColor3d(1,1,1);
-	glTexImage2D(&GL_TEXTURE_2D,
-		0,
-		&GL_RGB,
-		$txd,
-		$tyd,
-		0,
-		&GL_RGB,
-		&GL_FLOAT,
-		${$p->get_dataref()}
-	);
+         glTexImage2D_s(GL_TEXTURE_2D, 0, GL_RGB, $txd, $tyd, 0, GL_RGB, GL_FLOAT, $p->get_dataref());
 	 glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST );
 	    glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST );
 	       glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT );
 	          glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT );
 
-	glDisable(&GL_LIGHTING);
+	glDisable(GL_LIGHTING);
 	glNormal3d(0,0,1);
-	glEnable(&GL_TEXTURE_2D);
-	glBegin(&GL_QUADS);
+	glEnable(GL_TEXTURE_2D);
+	glBegin(GL_QUADS);
 	my @texvert = (
 		[0,0],
 		[$xd/$txd, 0],
@@ -543,8 +612,8 @@ sub PDL::Graphics::TriD::Image::gdraw {
 		&glVertex3f($vert->slice(":,($_)")->list);
 	}
 	glEnd();
-	glEnable(&GL_LIGHTING);
-	glDisable(&GL_TEXTURE_2D);
+	glEnable(GL_LIGHTING);
+	glDisable(GL_TEXTURE_2D);
 }
 
 sub PDL::Graphics::TriD::SimpleController::togl {
@@ -564,7 +633,11 @@ sub PDL::Graphics::TriD::SimpleController::togl {
 #
 #
 package PDL::Graphics::TriD::Window;
-use PDL::Graphics::OpenGL;
+
+use OpenGL qw/ :glfunctions :glconstants :glxconstants /;
+use OpenGL::GLUT qw( :all );
+use PDL::Graphics::OpenGL::Perl::OpenGL;
+
 use base qw/PDL::Graphics::TriD::Object/;
 use fields qw/Ev Width Height Interactive _GLObject 
               _ViewPorts _CurrentViewPort /;
@@ -574,20 +647,22 @@ use strict;
 
 sub gdriver {
   my($this, $options) = @_;
+  
+  print "GL gdriver...\n" if($PDL::debug_trid);
 
   if(defined $this->{_GLObject}){
 	 print "WARNING: Graphics Driver already defined for this window \n";
 	 return;
   }
-  my @db = &GLX_DOUBLEBUFFER;
+  my @db = OpenGL::GLX_DOUBLEBUFFER;
 
   if($PDL::Graphics::TriD::offline) {$options->{x} = -1; @db=()}
 
-  $options->{attributes} = [&GLX_RGBA, @db,
-			    &GLX_RED_SIZE,1,
-			    &GLX_GREEN_SIZE,1,
-			    &GLX_BLUE_SIZE,1,
-			    &GLX_DEPTH_SIZE,1,
+  $options->{attributes} = [GLX_RGBA, @db,
+			    GLX_RED_SIZE,1,
+			    GLX_GREEN_SIZE,1,
+			    GLX_BLUE_SIZE,1,
+			    GLX_DEPTH_SIZE,1,
 			    # Alpha size?
 			   ] unless defined $options->{attributes};
     
@@ -598,26 +673,49 @@ sub gdriver {
 
 
   print "STARTING OPENGL $options->{width} $options->{height}\n" if($PDL::Graphics::TriD::verbose);
-  
+
+  print "gdriver: Calling OpengGL::OO($options)...\n" if ($PDL::debug_trid);
+
   $this->{_GLObject}= new PDL::Graphics::OpenGL::OO($options);
 
+  if (exists $this->{_GLObject}->{glutwindow}) {
+     if ($PDL::debug_trid) {
+        print "gdriver: Got OpenGL::OO object(GLUT window ID# " . $this->{_GLObject}->{glutwindow} . ")\n";
+     }
+     $this->{_GLObject}->{winobjects}->[$this->{_GLObject}->{glutwindow}] = $this;      # circular ref
+  }
+
 #glpOpenWindow(%$options);
   
+  print "gdriver: Calling glClearColor...\n" if ($PDL::debug_trid);
   glClearColor(0,0,0,1);
-  my $lb =  $this->{_GLObject}->glpRasterFont(
-						($ENV{PDL_3D_FONT} or "5x8"),0,256);
-  $PDL::Graphics::TriD::GL::fontbase = $lb;
-  #	glDisable(&GL_DITHER);
-  glShadeModel (&GL_FLAT);
-  glEnable(&GL_DEPTH_TEST);
-  glEnable(&GL_NORMALIZE);
-  glEnable(&GL_LIGHTING);
-  glEnable(&GL_LIGHT0);
-  glLightModeli(&GL_LIGHT_MODEL_TWO_SIDE, &GL_TRUE);
+
+  print "gdriver: Calling glpRasterFont...\n" if ($PDL::debug_trid);
+  if ( $this->{_GLObject}->{window_type} eq 'glut' ) {
+     print STDERR "gdriver: window_type => 'glut' so not actually setting the rasterfont\n" if ($PDL::debug_trid);
+     eval '$PDL::Graphics::TriD::GL::fontbase = GLUT_BITMAP_8_BY_13';
+  } else {
+     # NOTE: glpRasterFont() will die() if the requested font cannot be found
+     #       The new POGL+GLUT TriD implementation uses the builtin GLUT defined
+     #       fonts and does not have this failure mode.
+     
+     my $lb =  eval { $this->{_GLObject}->glpRasterFont( ($ENV{PDL_3D_FONT} or "5x8"), 0, 256 ) };
+     if ( $@ ) {
+        die "glpRasterFont: unable to load font '%s', please set PDL_3D_FONT to an existing X11 font.";
+     }
+     $PDL::Graphics::TriD::GL::fontbase = $lb
+  }
+  #	glDisable(GL_DITHER);
+  glShadeModel (GL_FLAT);
+  glEnable(GL_DEPTH_TEST);
+  glEnable(GL_NORMALIZE);
+  glEnable(GL_LIGHTING);
+  glEnable(GL_LIGHT0);
+  glLightModeli(GL_LIGHT_MODEL_TWO_SIDE, GL_TRUE);
   
   # Will this bring us trouble?
   #	if(defined *PDL::Graphics::TriD::GL::Window::glPolygonOffsetEXT{CODE}) {
-#  glEnable(&GL_POLYGON_OFFSET_EXT);
+#  glEnable(GL_POLYGON_OFFSET_EXT);
 #  glPolygonOffsetEXT(0.0000000000001,0.000002);
   #	}
   
@@ -626,7 +724,7 @@ sub gdriver {
 #  $this->reshape($options->{width},$options->{height});
 
   my $light = pack "f*",1.0,1.0,1.0,0.0;
-  glLightfv(&GL_LIGHT0,&GL_POSITION,$light);
+  glLightfv_s(GL_LIGHT0,GL_POSITION,$light);
 
   glColor3f(1,1,1);
   
@@ -641,15 +739,15 @@ sub gdriver {
 }
 
 sub ev_defaults{
-  return {	&ConfigureNotify => \&doconfig,
-				&MotionNotify => \&domotion,
+  return {	ConfigureNotify => \&doconfig,
+				MotionNotify => \&domotion,
 			}
 }
 
 sub reshape {
 	my($this,$x,$y) = @_;
-   my $pw = $this->{Width};
-   my $ph = $this->{Height};
+	my $pw = $this->{Width};
+	my $ph = $this->{Height};
 	$this->{Width} = $x; $this->{Height} = $y;
 
 	for my $vp (@{$this->{_ViewPorts}}){
@@ -657,8 +755,8 @@ sub reshape {
 	  my $nx0 = $vp->{X0} + ($x-$pw) * $vp->{X0}/$pw;
 	  my $nh = $vp->{H} + ($y-$ph) * $vp->{H}/$ph;
 	  my $ny0 = $vp->{Y0} + ($y-$ph) * $vp->{Y0}/$ph;
+	  print "reshape: resizing viewport to $nx0,$ny0,$nw,$nh\n" if($PDL::Graphics::TriD::verbose);
 	  $vp->resize($nx0,$ny0,$nw,$nh);
-	  
 	}
 
 }
@@ -682,31 +780,48 @@ sub twiddle {
 	 wpic($this->read_picture(),"PDL_$PDL::Graphics::TriD::offlineindex.jpg");
 	 return;
   }
-  if($getout and $dontshow) {
-	 if(!$this->{_GLObject}->XPending()) {return}
+  if ($getout and $dontshow) {
+	 if ( !$this->{_GLObject}->XPending() ) {
+            return;
+         }
   }
   if(!defined $getout) {
-	 $getout = not $PDL::Graphics::TriD::keeptwiddling;
+	 $getout = not ($PDL::Graphics::TriD::keeptwiddling && $PDL::Graphics::TriD::keeptwiddling);
   }
   
   $this->display();
  TWIDLOOP: while(1) {
-	 #		print "EVENT!\n";
+   print "EVENT!\n" if($PDL::Graphics::TriD::verbose);
 	 my $hap = 0;
 	 my $gotev = 0;
 
-	 if($this->{_GLObject}->XPending() or !$getout) {
-		@e = $this->{_GLObject}->glpXNextEvent();
-		$gotev=1;
-	 }
+         # Run a MainLoop event if GLUT windows
+         # this pumps the system allowing callbacks to populate
+         # the fake XEvent queue.
+         #
+         glutMainLoopEvent() if $this->{_GLObject}->{window_type} eq 'glut' and not $this->{_GLObject}->XPending();
+
+         if ($this->{_GLObject}->XPending() or !$getout) {
+            @e = $this->{_GLObject}->glpXNextEvent();
+            $gotev=1;
+         }
+   print "e= ".join(",",@e)."\n" if($PDL::Graphics::TriD::verbose);
+	
 	 if(@e){
-		if($e[0] == &VisibilityNotify or $e[0] == &Expose) {
+		if ($e[0] == VisibilityNotify || $e[0] == Expose) {
 		  $hap = 1;
-		}elsif($e[0] == &ConfigureNotify) {
+		} elsif ($e[0] == ConfigureNotify) {
 		  print "CONFIGNOTIFE\n" if($PDL::Graphics::TriD::verbose);
 		  $this->reshape($e[1],$e[2]);
 		  $hap=1;
-		}elsif($e[0] == &KeyPress) {
+		} elsif ($e[0] == DestroyNotify) {
+		  print "DESTROYNOTIFE\n" if $PDL::Graphics::TriD::verbose;
+		  $quit = 1;
+		  $hap=1;
+		  undef $this->{_GLObject};
+		  $PDL::Graphics::TriD::cur = $PDL::Graphics::TriD::current_window = undef;
+		  last TWIDLOOP;
+		} elsif($e[0] == KeyPress) {
 		  print "KEYPRESS: '$e[1]'\n" if($PDL::Graphics::TriD::verbose);
 		  if((lc $e[1]) eq "q") {
 			 $quit = 1;
@@ -725,7 +840,7 @@ sub twiddle {
 		#			print "HANDLING $this->{EHandler}\n";
 		foreach my $vp (@{$this->{_ViewPorts}}) {
 		  if(defined($vp->{EHandler})) {
-			 $hap += $vp->{EHandler}->event(@e);
+			 $hap += $vp->{EHandler}->event(@e) || 0;
 		  }
 		}
 	 }
@@ -735,18 +850,14 @@ sub twiddle {
 		}
 		if($getout) {last TWIDLOOP}
 	 }
-	 undef @e;
+	 @e = ();
   }
   print "STOPTWIDDLE\n" if($PDL::Graphics::TriD::verbose);
   return $quit;
 }
 
-
-
-
 sub setlist { my($this,$list) = @_;
 	$this->{List} = $list;
-
 }
 
 # Resize window.
@@ -764,6 +875,14 @@ sub domotion {
 sub display {
   my($this) = @_;
 
+  return unless defined($this);
+
+  # set GLUT context to current window (for multiwindow support
+  if ( $this->{_GLObject}->{window_type} eq 'glut' ) {
+     glutSetWindow($this->{_GLObject}->{glutwindow});
+  }
+
+  print "display: calling glClear()\n" if ($PDL::Graphics::TriD::verbose);
   glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);
   glMatrixMode(GL_MODELVIEW);
 
@@ -777,7 +896,7 @@ sub display {
 
 
 	 if($vp->{Transformer}) {
-		print "Transforming viewport!\n" if($PDL::Graphics::TriD::verbose);
+		print "display: transforming viewport!\n" if ($PDL::Graphics::TriD::verbose);
 		$vp->{Transformer}->togl();
 	 }
 
@@ -790,7 +909,14 @@ sub display {
 
   }
 
-  $this->{_GLObject}->glXSwapBuffers();
+  print "display: SwapBuffers() call on return\n" if ($PDL::Graphics::TriD::verbose);
+  if ( $this->{_GLObject}->{window_type} eq 'glut' ) {  # need to make method call
+    glutSwapBuffers();
+  } elsif ( $this->{_GLObject}->{window_type} eq 'x11' ) {  # need to make method call
+    $this->{_GLObject}->glXSwapBuffers();
+  } else {
+    print "display: got object with inconsistent _GLObject info\n";
+  }
 #  $this->{Angle}+= 3;
 }
 
@@ -799,68 +925,91 @@ sub read_picture {
 	my($this) = @_;
 	my($w,$h) = @{$this}{qw/Width Height/};
 	my $res = PDL->zeroes(PDL::byte,3,$w,$h);
-	glPixelStorei(&GL_UNPACK_ALIGNMENT,1);
-	glPixelStorei(&GL_PACK_ALIGNMENT,1);
-	glReadPixels(0,0,$w,$h,&GL_RGB,&GL_UNSIGNED_BYTE,
-		${$res->get_dataref});
+	glPixelStorei(GL_UNPACK_ALIGNMENT,1);
+	glPixelStorei(GL_PACK_ALIGNMENT,1);
+        glReadPixels_s(0,0,$w,$h,GL_RGB,GL_UNSIGNED_BYTE,$res->get_dataref);
 	return $res;
 }
 
+######################################################################
+######################################################################
+# EVENT HANDLER MINIPACKAGE FOLLOWS!
 
 package PDL::Graphics::TriD::EventHandler;
-use PDL::Graphics::OpenGL qw/MotionNotify ButtonPress ButtonRelease 
-                             Button1Mask Button2Mask Button3Mask/;
+
+use OpenGL qw(
+  ConfigureNotify MotionNotify DestroyNotify
+  ButtonPress ButtonRelease Button1Mask Button2Mask Button3Mask
+);
+use PDL::Graphics::OpenGL::Perl::OpenGL;
+
 use fields qw/X Y Buttons VP/;
 use strict;
 sub new {
   my $class = shift;
   my $vp = shift;
   no strict 'refs';
-  my $self = bless [ \%{"$class\::FIELDS"}], $class;
+  my $self = fields::new($class);
   $self->{X} = -1;
   $self->{Y} = -1;
   $self->{Buttons} = [];
   $self->{VP} = $vp;
+
   $self;
 }
 
 sub event {
   my($this,$type,@args) = @_;
 
-  #  print "EH: ",ref($this)," $type\n" if($PDL::Graphics::TriD::verbose);
+  print "EH: ",ref($this)," $type (",join(",",@args),")\n" if($PDL::Graphics::TriD::verbose);
   my $retval;
-  if($type == &MotionNotify) {
+
+  if($type == MotionNotify) {
 	 my $but = -1;
-  
+	 
   SWITCH: { 
 		$but = 0, last SWITCH if ($args[0] & (Button1Mask));
 		$but = 1, last SWITCH if ($args[0] & (Button2Mask));
 		$but = 2, last SWITCH if ($args[0] & (Button3Mask));
+		print "No button pressed...\n" if($PDL::Graphics::TriD::verbose);
 		goto NOBUT;
 	 }
 
-#	 print "MOTION $but $args[0]\n";
+	 print "MOTION $but $args[0]\n" if($PDL::Graphics::TriD::verbose);
 	 if($this->{Buttons}[$but]) {
 		if($this->{VP}->{Active}){
+		  print "calling ".($this->{Buttons}[$but])."->mouse_moved ($this->{X},$this->{Y},$args[1],$args[2])...\n" if($PDL::Graphics::TriD::verbose);
 		  $retval = $this->{Buttons}[$but]->mouse_moved(
-														  $this->{X},$this->{Y},
-														  $args[1],$args[2]);
+								$this->{X},$this->{Y},
+								$args[1],$args[2]);
 		}
 	 }
 	 $this->{X} = $args[1]; $this->{Y} = $args[2];
   NOBUT:
-  } elsif($type == &ButtonPress) {
+
+       } elsif($type == ButtonPress) {
+
 	 my $but = $args[0]-1;
-#	 print "BUTTONPRESS $but\n";
+	 print "BUTTONPRESS $but\n" if($PDL::Graphics::TriD::verbose);
 	 $this->{X} = $args[1]; $this->{Y} = $args[2];
 	 $retval = $this->{Buttons}[$but]->ButtonPress($args[1],$args[2]) 
-		if($this->{Buttons}[$but]);
-  } elsif($type == &ButtonRelease) {
+	   if($this->{Buttons}[$but]);
+
+       } elsif($type == ButtonRelease) {
+
 	 my $but = $args[0]-1;
-#	 print "BUTTONRELEASE $but\n";
+	 print "BUTTONRELEASE $but\n" if($PDL::Graphics::TriD::verbose);
 	 $retval = $this->{Buttons}[$but]->ButtonRelease($args[1],$args[2]) 
-		if($this->{Buttons}[$but]);
-  }
+	   if($this->{Buttons}[$but]);
+
+       } elsif($type== ConfigureNotify) {
+
+	 # Kludge to force reshape of the viewport associated with the window -CD
+	 print "ConfigureNotify (".join(",",@args).")\n" if($PDL::Graphics::TriD::verbose);
+	 print "viewport is $this->{VP}\n" if($PDL::Graphics::TriD::verbose);
+#	 $retval = $this->reshape(@args);
+
+       }
   $retval;
 }
 
@@ -869,18 +1018,20 @@ sub set_button {
 	$this->{Buttons}[$butno] = $act;
 }
 
-  
-
+######################################################################
+######################################################################
+# VIEWPORT MINI_PACKAGE FOLLOWS!
 
 package PDL::Graphics::TriD::ViewPort;
 use base qw/PDL::Graphics::TriD::Object/;
 use fields qw/X0 Y0 W H Transformer EHandler Active ResizeCommands 
               DefMaterial AspectRatio Graphs/;
 
-use PDL::Graphics::OpenGL;
+use OpenGL qw/ :glfunctions :glconstants :glufunctions /;
+use OpenGL::GLUT qw( :all );
+use PDL::Graphics::OpenGL::Perl::OpenGL;
 use PDL::Graphics::OpenGLQ;
 
-
 sub highlight {
   my ($vp) = @_;
 
@@ -893,7 +1044,7 @@ sub highlight {
 
   $colors = PDL->ones(3,5);
 
-  glDisable(&GL_LIGHTING);
+  glDisable(GL_LIGHTING);
   
   glMatrixMode(GL_MODELVIEW);
   glLoadIdentity();
@@ -907,7 +1058,7 @@ sub highlight {
 
   glLineWidth(1);
 		
-  glEnable(&GL_LIGHTING);
+  glEnable(GL_LIGHTING);
 
 }
 
@@ -918,6 +1069,17 @@ sub do_perspective {
 
 	print "do_perspective ",$this->{W}," ",$this->{H} ,"\n" if($PDL::Graphics::TriD::verbose);
 
+	if($PDL::Graphics::TriD::verbose>1){
+	  my ($i,$package,$filename,$line);
+          $i = 0;
+	  do { 
+	    ($package,$filename,$line) = caller($i++);
+	    print "$package ($filename, line $line)\n";
+	  } while($package);
+	  print "\n";
+	}
+	      
+
         unless($this->{W}>0 and $this->{H}>0) {return;}
 #	if($this->{W}==0 or $this->{H}==0) {return;}
 	$this->{AspectRatio} = (1.0*$this->{W})/$this->{H};
