@@ -287,10 +287,10 @@ a file, so I figured I would write a small bit of documentation on it.
 
 =head1 Bad Value Support
 
-As of PDL-2.4.8, L<PDL::IO::FlexRaw|PDL::IO::FlexRaw> has support for reading and writing
+As of PDL-2.4.8, L<PDL::IO::FlexRaw> has support for reading and writing
 pdls with L<bad|PDL::Bad> values in them.
 
-On C<writeflex>, a piddle
+On C<writeflex>, an ndarray
 argument with C<< $pdl->badflag == 1 >> will have the keyword/token "badvalue"
 added to the header file after the dimension list and an additional token
 with the bad value for that pdl if C<< $pdl->badvalue != $pdl->orig_badvalue >>.
@@ -315,7 +315,7 @@ reference the current C<$hdr> looks like this:
 
     $badpdl = readflex('badpdl', [$hdr]);
 
-If you use bad values and try the new L<PDL::IO::FlexRaw|PDL::IO::FlexRaw> bad value
+If you use bad values and try the new L<PDL::IO::FlexRaw> bad value
 support, please let us know via the perldl mailing list.
 Suggestions and feedback are also welcome.
 
@@ -334,63 +334,36 @@ Documentation contributions copyright (C) David Mertens, 2010.
 =cut
 
 package PDL::IO::FlexRaw;
-
-BEGIN {
-   our $have_file_map = 0;
-
-   eval "use File::Map 0.57 qw(map_file)";
-   $have_file_map = 1 unless $@;
-}
-
+use strict;
+use warnings;
 use PDL;
 use Exporter;
-use FileHandle;
 use PDL::Types ':All';
-use PDL::IO::Misc qw(bswap2 bswap4 bswap8);
+use PDL::IO::Misc qw(bswap4);
 
-@PDL::IO::FlexRaw::ISA = qw/Exporter/;
-
-@EXPORT = qw/writeflex writeflexhdr readflex mapflex/;
+our @ISA = qw/Exporter/;
+our @EXPORT = qw/writeflex writeflexhdr readflex mapflex/;
 
 # Cast type numbers in concrete, for external file's sake...
-%flexnames = ( map {(typefld($_,'numval') => typefld($_,'ioname'))}
+my %flexnames = ( map {(typefld($_,'numval') => typefld($_,'ioname'))}
 	       typesrtkeys());
-%flextypes = ( map {(typefld($_,'ioname') => typefld($_,'numval'),
+my %flextypes = ( map {(typefld($_,'ioname') => typefld($_,'numval'),
 		     typefld($_,'numval') => typefld($_,'numval'),
 		     lc typefld($_,'ppsym') =>  typefld($_,'numval'),
 		    )}
 	       typesrtkeys());
-%flexswap = ( map {my $val = typefld($_,'numval');
+my %flexswap = ( map {my $val = typefld($_,'numval');
 		   my $nb = PDL::Core::howbig($val);
 		   ($val =>  $nb > 1 ? "bswap$nb" : undef)}
 	      typesrtkeys());
 
-# use Data::Dumper;
-# print Dumper \%flexnames;
-# print Dumper \%flextypes;
-# print Dumper \%flexswap;
-
-# %flexnames = (
-#    $PDL_B => 'byte', $PDL_S => 'short',
-#    $PDL_US => 'ushort', $PDL_L => 'long',
-#    $PDL_F => 'float', $PDL_D => 'double');
-
-# %flextypes = (
-# 'byte'   => $PDL_B, '0' => $PDL_B, 'b' => $PDL_B, 'c' => $PDL_B,
-# 'short'  => $PDL_S, '1' => $PDL_S, 's' => $PDL_S,
-# 'ushort' => $PDL_US,'2' => $PDL_US,'u' => $PDL_US,
-# 'long'   => $PDL_L, '3' => $PDL_L, 'l' => $PDL_L,
-# 'float'  => $PDL_F, '4' => $PDL_F, 'f' => $PDL_F,
-# 'double' => $PDL_D, '5' => $PDL_D, 'd' => $PDL_D
-# );
-
-$PDL::IO::FlexRaw::verbose = 0;
-$PDL::IO::FlexRaw::writeflexhdr = defined($PDL::FlexRaw::IO::writeflexhdr) ? $PDL::FlexRaw::IO::writeflexhdr : 0;
+our $verbose = 0;
+our $writeflexhdr //= 0;
 
 sub _read_flexhdr {
     my ($hname) = @_;
-    my $hfile = new FileHandle "$hname"
-	or barf "Couldn't open '$hname' for reading";
+    open my $hfile, $hname
+	or barf "Couldn't open '$hname' for reading: $!";
     binmode $hfile;
     my ($newfile) = 1;
     my ($tid, @str);
@@ -401,7 +374,7 @@ sub _read_flexhdr {
     seek $hfile, 0, 0;  # reset file pointer to beginning
  ITEM:
  while (!eof($hfile)) {
-    my (@dims) = (); my ($ndims) = -1, ($mode) = -2;
+    my ($ndims, $mode, @dims) = (-1, -2);
     my ($have_badvalue) = undef;
     my ($badvalue) = undef;
     LINE:
@@ -482,33 +455,33 @@ sub _read_flexhdr {
 }
 
 sub readchunk {
-    my ($d, $pdl, $len, $name) = @_;
+    my ($d, $pdl, $len, $name, $offset) = @_;
     my ($nread);
     print "Reading $len at $offset from $name\n"
-      if $PDL::IO::FlexRaw::verbose;
+      if $verbose;
     ($nread = read($d, ${$pdl->get_dataref}, $len)) == $len
 	or barf "Couldn't read $len bytes at offset $offset from '$name', got $nread";
     $pdl->upd_data();
-    $offset += $len;
-    return 1;
+    $len;
 }
 
+our $flexmapok;
 sub myhandler {
     $flexmapok = 0;
     barf "Data out of alignment, can't map further\n";
-    die;
 }
 
 sub mapchunk {
-    my ($orig, $pdl, $len, $name) = @_;
+    my ($orig, $pdl, $len, $name, $offset) = @_;
     # link $len at $offset from $orig to $pdl.
     # print "linking $len bytes from $offset\n";
+    $pdl->freedata;
     $pdl->set_data_by_offset($orig,$offset);
-    local ($flexmapok)=1;
+    local $flexmapok=1;
     local $SIG{BUS} = \&myhandler unless $^O =~ /MSWin32/i;
     local $SIG{FPE} = \&myhandler;
     eval {$pdl->clump(-1)->at(0)};
-    $offset += $len;
+    $_[4] += $len; # mutate input
     $flexmapok;
 }
 
@@ -517,7 +490,7 @@ sub readflex {
 		if $#_ > 1;
     my ($name,$h) = @_;
     my ($hdr, $pdl, $len, @out, $chunk, $chunkread, $data);
-    local ($offset) = 0;
+    my $offset = 0;
     my ($newfile, $swapbyte, $f77mode, $zipt) = (1,0,0,0);
     my $d;
     # print("readflex: name is $name\n");
@@ -563,8 +536,8 @@ sub readflex {
 	  }
 	}
 	my ($size) = (stat $name)[7];
-	$d = new FileHandle $data
-	    or barf "Couldn't open '$data' for reading";
+	open $d, $data
+	    or barf "Couldn't open '$data' for reading: $!";
 	binmode $d;
 	$h = _read_flexhdr("$name.hdr")
 	    unless $h;
@@ -609,15 +582,12 @@ READ:
 			    ref $hdr->{Dims} ? @{$hdr->{Dims}} : $hdr->{Dims});
 	$len = length $ {$pdl->get_dataref};
 
-	&readchunk($d,$pdl,$len,$name) or last READ;
+	$offset += readchunk($d,$pdl,$len,$name, $offset);
 	$chunkread += $len;
 	if ($swapbyte) {
 	  my $method = $flexswap{$type};
 	  $pdl->$method if $method;
-# 	    bswap2($pdl) if $pdl->get_datatype == $PDL_S;
-# 	    bswap4($pdl) if $pdl->get_datatype == $PDL_L
-# 		|| $pdl->get_datatype == $PDL_F;
-# 	    bswap8($pdl) if $pdl->get_datatype == $PDL_D;
+# 	    $pdl->type->bswap($pdl);
 	}
 	if ($newfile && $f77mode) {
 	    if ($zipt || $swapbyte) {
@@ -655,14 +625,14 @@ READ:
 	if ($f77mode && $chunk->at == $chunkread) {
 	    $chunkread = 0;
 	    my ($check) = $chunk->copy;
-	    &readchunk($d,$check,4,$name) or last READ;
+	    $offset += readchunk($d,$check,4,$name,$offset);
 	    bswap4($check) if $swapbyte;
 	    if ($check->at ne $chunk->at) {
 		barf "F77 file format error for $check cf $chunk";
 		last READ;
 	    }
 	    if (!eof($d)) {
-		&readchunk($d,$chunk,4,$name) or last READ;
+		$offset += readchunk($d,$chunk,4,$name,$offset);
 		bswap4($chunk) if $swapbyte;
 	    } else {
 		last READ;
@@ -682,7 +652,7 @@ sub mapflex {
     my (%opts) = ( 'ReadOnly' => 0, 'Creat' => 0, 'Trunc' => 0 );
 
     my ($hdr, $d, $pdl, $len, @out, $chunk, $chunkread);
-    local ($offset) = 0;
+    my $offset = 0;
     my ($newfile, $swapbyte, $f77mode, $zipt) = (1,0,0,0);
 
     foreach (@_) {
@@ -737,26 +707,14 @@ sub mapflex {
     # print "Mapping total size $size\n";
     # use Data::Dumper;
     # print "Options: ", Dumper(\%opts), "\n";
-    if ($have_file_map and not defined($PDL::force_use_mmap_code) ) {
-       $d->set_data_by_file_map($name,
-                            $size,
-                            1,
-                            ($opts{ReadOnly}?0:1),
-                            ($opts{Creat}?1:0),
-                            (0644),
-                            ($opts{Creat} || $opts{Trunc} ? 1:0)
-                         );
-    } else {
-       warn "mapflex: direct mmap support being deprecated, please install File::Map\n";
-       $d->set_data_by_mmap($name,
-                            $size,
-                            1,
-                            ($opts{ReadOnly}?0:1),
-                            ($opts{Creat}?1:0),
-                            (0644),
-                            ($opts{Creat} || $opts{Trunc} ? 1:0)
-                         );
-    }
+    $d->set_data_by_file_map($name,
+        $size,
+        1,
+        ($opts{ReadOnly}?0:1),
+        ($opts{Creat}?1:0),
+        (0644),
+        ($opts{Creat} || $opts{Trunc} ? 1:0)
+    );
 READ:
     foreach $hdr (@$h) {
 		my ($type) = $hdr->{Type};
@@ -783,7 +741,7 @@ READ:
 					ref $hdr->{Dims} ? @{$hdr->{Dims}} : $hdr->{Dims});
 		$len = length $ {$pdl->get_dataref};
 
-		&mapchunk($d,$pdl,$len,$name) or last READ;
+		&mapchunk($d,$pdl,$len,$name,$offset) or last READ;
 		$chunkread += $len;
 		if ($newfile && $f77mode) {
 			if ($opts{Creat}) {
@@ -804,7 +762,7 @@ READ:
 		if ($f77mode && $chunk->at == $chunkread) {
 			$chunkread = 0;
 			my ($check) = $chunk->copy;
-			&mapchunk($d,$check,4,$name) or last READ;
+			&mapchunk($d,$check,4,$name,$offset) or last READ;
 			if ($opts{Creat}) {
 				$check->set(0,$size-8);
 				} else {
@@ -813,7 +771,7 @@ READ:
 					last READ;
 				}
 			}
-			barf "Will only map first f77 data statement" if ($offset < $size);
+			barf "Will only map first f77 data statement" if $offset < $size;
 			last READ;
 		}
     }
@@ -837,11 +795,11 @@ sub writeflex {
       barf $usage if ref $name;
       $isname = 1;
       my $modename = ($name =~ /^[+]?[><|]/) ? $name : ">$name";
-      $d = new FileHandle $modename
-         or barf "Couldn't open '$name' for writing";
+      open $d, $modename
+         or barf "Couldn't open '$name' for writing: $!";
       binmode $d;
    }
-   foreach $pdl (@_) {
+   foreach my $pdl (@_) {
       barf $usage if ! ref $pdl;
       # print join(' ',$pdl->getndims,$pdl->dims),"\n";
       push @{$hdr}, {
@@ -851,11 +809,11 @@ sub writeflex {
          BadFlag => $pdl->badflag,
          BadValue => (($pdl->badvalue == $pdl->orig_badvalue) ? undef : $pdl->badvalue),
       };
-      print $d $ {$pdl->get_dataref};
+      print $d ${$pdl->get_dataref};
    }
    if (defined wantarray) {
       # list or scalar context
-      writeflexhdr($name, $hdr) if $isname and $PDL::IO::FlexRaw::writeflexhdr;
+      writeflexhdr($name, $hdr) if $isname and $writeflexhdr;
       return $hdr;
    } else {
       # void context so write header file
@@ -868,8 +826,8 @@ sub writeflexhdr {
     barf 'Usage writeflex("filename", $hdr)' if $#_!=1 || !ref $_[1];
     my($name) = shift; my ($hdr) = shift;
     my $hname = "$name.hdr";
-    my $h = new FileHandle ">$hname"
-	or barf "Couldn't open '$hname' for writing";
+    open my $h, '>', $hname
+        or barf "Couldn't open '$hname' for writing: $!";
     binmode $h;
     print $h
 	"# Output from PDL::IO::writeflex, data in $name\n";
@@ -887,5 +845,3 @@ sub writeflexhdr {
 }
 
 1;
-
-
